const PAYMENT_METHOD = 'revolut';

/**
 * Check shipping information
 * @param {Object} input
 * @return {boolean}
 */
function isValidData(input) {
    return Boolean(
        input.billing_first_name &&
        input.billing_last_name &&
        input.billing_email &&
        input.billing_country &&
        input.billing_city &&
        input.billing_address_1 &&
        input.billing_postcode &&
        input.billing_phone
    );
}

/**
 * Get payment data
 * @param {Object} input
 * @return {{}}
 */
function getPaymentData(input) {
    let data = {};

    data.name = input.billing_first_name + ' ' + input.billing_last_name;
    data.email = input.billing_email;
    data.phone = input.billing_tel;

    data.billingAddress = {
        countryCode: input.billing_country,
        region: input.billing_state,
        city: input.billing_city,
        streetLine1: input.billing_address_1,
        streetLine2: input.billing_address_2,
        postcode: input.billing_postcode,
    };

    if (input.ship_to_different_address) {
        data.shippingAddress = {
            countryCode: input.shipping_country,
            region: input.shipping_state,
            city: input.shipping_city,
            streetLine1: input.shipping_address_1,
            streetLine2: input.shipping_address_2,
            postcode: input.shipping_postcode,
        };
    }

    return data;
}

jQuery(function ($) {
    let $body = $(document.body);
    let $form = $('form.woocommerce-checkout');

    let isSubmitting = false;
    let isSubmitted = false;
    let cardStatus = null;

    /**
     * Display error message
     * @param {string} messages
     */
    function displayError(messages) {
        $('.woocommerce-NoticeGroup-checkout, .woocommerce-error, .woocommerce-message').remove();

        let normalized = [].concat(messages);

        if (normalized.length > 0) {
            $form.prepend(
                '<div class="woocommerce-NoticeGroup woocommerce-NoticeGroup-checkout">' +
                '<ul class="woocommerce-error">' +
                normalized.map(function (message) {
                    return '<li>' + message.message + '</li>';
                }) +
                '</ul>' +
                '</div>'
            );

            $body.trigger('checkout_error');
        }
    }

    /**
     * Start processing
     */
    function startProcessing() {
        $.blockUI({message: null, overlayCSS: {background: '#fff', opacity: 0.6}});
        isSubmitting = true;
    }

    /**
     * Stop processing
     */
    function stopProcessing() {
        $.unblockUI();
        isSubmitting = false;
    }

    /**
     * Handle status change
     * @param {string} status
     */
    function handleStatusChange(status) {
        cardStatus = status;
    }

    /**
     * Handle validation
     * @param {string} messages
     */
    function handleValidation(messages) {
        displayError(messages);
    }

    /**
     * Handle cancel
     */
    function handleCancel() {
        stopProcessing();
    }

    /**
     * Handle error message
     * @param {string} message
     */
    function handleError(message) {
        stopProcessing();
        displayError(message);
    }

    /**
     * Handle if success
     */
    function handleSuccess() {
        isSubmitted = true;

        stopProcessing();
        $form.submit();
    }

    let instance = null;

    /**
     * Update revolut order
     */
    function handleUpdate() {
        let target = document.getElementById('woocommerce-revolut-card-element');

        if (target === null) {
            return;
        }

        let publicId = target.dataset.publicId;

        if (publicId === null) {
            return;
        }

        if (instance !== null) {
            instance.destroy();
        }

        instance = RevolutCheckout(publicId).createCardField({
            target: target,
            hidePostcodeField: true,
            onCancel: handleCancel,
            onStatusChange: handleStatusChange,
            onValidation: handleValidation,
            onError: handleError,
            onSuccess: handleSuccess,
        });
    }

    /**
     * Show validate error messages
     * @param errorMessage
     */
    function submitError(errorMessage) {
        $( '.woocommerce-NoticeGroup-checkout, .woocommerce-error, .woocommerce-message' ).remove();
        $form.prepend( '<div class="woocommerce-NoticeGroup woocommerce-NoticeGroup-checkout">' + errorMessage + '</div>' ); // eslint-disable-line max-len
        $form.removeClass( 'processing' ).unblock();
        $form.find( '.input-text, select, input:checkbox' ).trigger( 'validate' ).blur();
        var scrollElement = $( '.woocommerce-NoticeGroup-updateOrderReview, .woocommerce-NoticeGroup-checkout' );
        if ( ! scrollElement.length ) {
            scrollElement = $( '.form.checkout' );
        }
        $.scroll_to_notices( scrollElement );
        $( document.body ).trigger( 'checkout_error' );
    }
    /**
     * Submit order to API
     * @return {boolean}
     */
    function handleSubmit() {
        if (isSubmitting) return false;
        if (isSubmitted) return true;

        const json = $form.serializeArray().reduce(function (acc, item) {
            acc[item.name] = item.value;
            return acc;
        }, {});

        if (json.payment_method === PAYMENT_METHOD) {
            $.ajax({
                type: "POST",
                url: ajax.ajaxurl,
                data: {
                    action: 'validate_checkout_field',
                    json: json
                },

                success: function (result) {
                    try {
                        if (result.result == 'success') {
                            instance.submit(getPaymentData(json));

                            if (cardStatus !== null && cardStatus.completed) {
                                startProcessing();
                                $form.removeClass('.processing');
                                $form.trigger('submit');
                            }
                        } else if ('failure' === result.result) {
                            throw 'Result failure';
                        } else {
                            throw 'Invalid response';
                        }
                    } catch (err) {
                        // Add new errors
                        if (result.messages) {
                            submitError(result.messages);
                        }
                    }
                }
            });
        }
        return false;
    }

    $body.on('updated_checkout', handleUpdate);
    $form.on('checkout_place_order_revolut', handleSubmit);
});
