<?php
/**
 * Plugin Name: Revolut Gateway for WooCommerce
 * Plugin URI: https://woocommerce.com/products/revolut-payment-gateway/
 * Description: Receive payments using Revolut.
 * Author: Revolut
 * Author URI: http://revolut.com/
 * Version: 1.2.1
 * Requires at least: 4.4
 * Tested up to: 5.6
 * WC tested up to: 4.7.0
 * WC requires at least: 2.6
 *
 */
defined( 'ABSPATH' ) || exit;
define( 'REVOLUT_PATH', plugin_dir_path( __FILE__ ) );

/**
 * Init revolut
 */

function woocommerce_revolut_init() {
	if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
		return;
	}

	define( 'WC_GATEWAY_REVOLUT_VERSION', '1.2.1' );
	define( 'WC_REVOLUT_MAIN_FILE', __FILE__ );
	define( 'WC_REVOLUT_PLUGIN_URL', untrailingslashit( plugins_url( basename( plugin_dir_path( __FILE__ ) ), basename( __FILE__ ) ) ) );
	add_action( 'admin_enqueue_scripts', 'load_admin_scripts' );
	require_once( plugin_basename( 'includes/class-wc-gateway-revolut.php' ) );
	require_once( plugin_basename( 'includes/class-wc-gateway-revolut-privacy.php' ) );
	require_once( plugin_basename( 'includes/class-wc-revolut-validate-checkout.php' ) );
	load_plugin_textdomain( 'woocommerce-gateway-revolut', false, trailingslashit( dirname( plugin_basename( __FILE__ ) ) ) );
	add_filter( 'woocommerce_payment_gateways', 'woocommerce_revolut_add_gateway' );
	add_action( 'wp_enqueue_scripts', 'load_scripts' );
	add_action( 'wp_ajax_validate_checkout_field', 'validate_checkout_field' );
	add_action( 'wp_ajax_nopriv_validate_checkout_field', 'validate_checkout_field' );
}

add_action( 'plugins_loaded', 'woocommerce_revolut_init', 0 );

/**
 * Set up Revolut plugin links
 *
 * @param $links
 *
 * @return array
 */
function woocommerce_revolut_plugin_links( $links ) {
	$settings_url = add_query_arg(
		array(
			'page'    => 'wc-settings',
			'tab'     => 'checkout',
			'section' => 'wc_gateway_revolut',
		),
		admin_url( 'admin.php' )
	);

	$plugin_links = array(
		'<a href="' . esc_url( $settings_url ) . '">' . __( 'Settings', 'woocommerce-gateway-revolut' ) . '</a>',
		'<a href="https://www.woocommerce.com/my-account/tickets/">' . __( 'Support', 'woocommerce-gateway-revolut' ) . '</a>',
		'<a href="https://docs.woocommerce.com/document/revolut-payment-gateway/">' . __( 'Docs', 'woocommerce-gateway-revolut' ) . '</a>',
	);

	return array_merge( $plugin_links, $links );
}

add_filter( 'plugin_action_links_' . plugin_basename( __FILE__ ), 'woocommerce_revolut_plugin_links' );

/**
 * Add Revolut payment gateway
 *
 * @param $methods
 *
 * @return mixed
 */
function woocommerce_revolut_add_gateway( $methods ) {
	$methods[] = 'WC_Gateway_Revolut';

	return $methods;
}

/**
 * Create table to save Revolut Order
 */
register_activation_hook( __FILE__, 'woocommerce_revolut_create_tables' );
function woocommerce_revolut_create_tables() {
	global $wpdb;
	$charset_collate = $wpdb->get_charset_collate();
	$table_name      = $wpdb->prefix . 'wc_revolut_orders';

	$sql = "CREATE TABLE $table_name (
		order_id BINARY(16) NOT NULL PRIMARY KEY,
		public_id BINARY(16) NOT NULL UNIQUE,
		wc_order_id INTEGER NULL UNIQUE,
		created_date TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
		updated_date TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP ON UPDATE CURRENT_TIMESTAMP
	) $charset_collate;";

	require_once( ABSPATH . 'wp-admin/includes/upgrade.php' );
	dbDelta( $sql );
}

/**
 * Add script to switch between live & sandbox modes
 */
function load_admin_scripts() {
	wp_register_script( 'revolut-settings', plugins_url( 'assets/js/revolut-setting.js', WC_REVOLUT_MAIN_FILE ) );
	wp_enqueue_script( 'revolut-settings' );
}

/**
 * Add script & style checkout form
 */
function load_scripts() {
	wp_register_script( 'checkout-form', plugins_url( 'assets/js/checkout-form.js', WC_REVOLUT_MAIN_FILE ) );
	wp_enqueue_script( 'checkout-form' );
	wp_register_style( 'custom-style', plugins_url( 'assets/css/style.css', WC_REVOLUT_MAIN_FILE ) );
	wp_enqueue_style( 'custom-style' );
	wp_register_script( 'revolut-woocommerce', plugins_url( 'assets/js/revolut.js', WC_REVOLUT_MAIN_FILE ), array(
		'revolut-core',
		'jquery'
	), WC_GATEWAY_REVOLUT_VERSION, true );
	wp_localize_script( 'revolut-woocommerce', 'ajax', array( 'ajaxurl' => admin_url( 'admin-ajax.php' )));
	wp_enqueue_script( 'revolut-woocommerce' );
}

/**
 * Validate checkout fields
 */
function validate_checkout_field() {
	$json = $_POST['json'];
	$validate_checkout = new Revolut_Validate_Checkout();
	$errors = $validate_checkout->validate_checkout_shipping($json, new WP_Error());

	foreach ( $errors->errors as $code => $messages ) {
		$data = $errors->get_error_data( $code );
		foreach ( $messages as $message ) {
			wc_add_notice( $message, 'error', $data );
		}
	}
	$messages = wc_print_notices(true);
	$response = array(
		'result'   => !!$messages ? 'failure' : 'success',
		'messages' => $messages,
	);

	wp_send_json( $response );
}