<?php

/**
 * Revolut Payment Gateway
 *
 * Provides a Revolut Payment Gateway.
 *
 * @class  woocommerce_revolut
 * @package WooCommerce
 * @category Payment Gateways
 * @author Revolut
 */

class WC_Gateway_Revolut extends WC_Payment_Gateway_CC {

	/**
	 * Version
	 *
	 * @var string
	 */
	public $version;

	private $api_key;
	private $api_key_sandbox;
	private $base_url;
	private $api_url;

	private $enable_logging = true;

	/**
	 * Constructor
	 */
	public function __construct() {
		$this->version      = WC_GATEWAY_REVOLUT_VERSION;
		$this->id           = 'revolut';
		$this->method_title = __( 'Revolut', 'woocommerce-gateway-revolut' );
		$this->has_fields   = true;
		/* translators: 1: a href link 2: closing href */
		$this->method_description = sprintf( __( 'Accept card payments easily and securely via %1$sRevolut%2$s.', 'woocommerce-gateway-revolut' ), '<a href="https://www.revolut.com/business/online-payments">', '</a>' );
		$this->icon               = $this->get_icon();

		// Supported functionality
		$this->supports = array(
			'products',
			'tokenization',
			'add_payment_method'
		);

		$this->title           = $this->get_option( 'title' );
		$this->api_key_sandbox = $this->get_option( 'api_key_sandbox' );

		//get setting from old version
		if ( 'yes' === $this->get_option( 'sandbox' ) && $this->get_option( 'mode' ) == "" ) {
			$this->update_option( 'mode', 'sandbox' );
			$this->update_option( 'api_key_sandbox', $this->get_option( 'api_key' ) );
			$this->update_option( 'api_key', '' );
		} else if ( 'yes' != $this->get_option( 'sandbox' ) && $this->get_option( 'mode' ) == "" ) {
			$this->update_option( 'mode', 'live' );
			$this->update_option( 'api_key', $this->get_option( 'api_key' ) );
		}

		$this->api_key  = $this->get_option( 'mode' ) == "sandbox" ? $this->api_key_sandbox : $this->get_option( 'api_key' );
		$this->base_url = $this->get_option( 'mode' ) == "sandbox" ? 'https://sandbox-merchant.revolut.com' : 'https://merchant.revolut.com';

		$this->api_url = $this->base_url . '/api/1.0';

		$this->init_form_fields();
		$this->init_settings();

		$this->description = $this->get_option( 'description' );

		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array(
			$this,
			'process_admin_options'
		) );
		add_action( 'admin_notices', array( $this, 'admin_notices' ) );
		add_action( 'wp_enqueue_scripts', array( $this, 'revolut_enqueue_scripts' ) );
		add_action( 'admin_notices', array( $this, 'add_revolut_description' ) );
	}

	/**
	 * Output Revolut description.
	 *
	 * @since 2.2.0
	 */
	function add_revolut_description() {
		$is_revolut_section = ($_GET['page'] == 'wc-settings' && ($_GET['section'] == 'revolut' || $_GET['section'] == 'wc_gateway_revolut'));

		if ( $is_revolut_section ) {
			if ( $this->api_key == "" ) {
				?>
                <div class="notice notice-info sf-notice-nux is-dismissible" xmlns="" id="revolut_notice">
                    <div class="notice-content">
                        <p>
                            Welcome to the <b>Revolut Gateway for Woocommerce plugin!</b>
                        </p>
                        <p>
                            To start accepting payments from your customers at great rates, you'll need to follow three
                            simple steps:
                        </p>
                        <ul style="list-style-type: disc; margin-left: 50px;">
                            <li>
                                <a href="https://business.revolut.com/signup">Sign up for Revolut Business</a> if you
                                don't
                                have an account already
                            </li>
                            <li>
                                Once your Revolut Business account has been approved, <a
                                        href="https://business.revolut.com/merchant">apply for a Merchant Account</a>
                            </li>
                            <li>
                                <a href="https://business.revolut.com/merchant/api">Get your Production API key</a> and
                                paste it in the corresponding field below
                            </li>
                        </ul>
                        <p>
                            <a href="https://www.revolut.com/business/online-payments">Find out more</a> about why
                            accepting
                            payments through Revolut is the right decision for your business.
                        </p>
                        <p>
                            If you'd like to know more about how to configure this plugin for your needs, <a
                                    href="https://developer.revolut.com/docs/accept-payments/#plugins-woocommerce">check
                                out
                                our documentation</a>.
                        </p>
                    </div>
                </div>
				<?php
			} else {
				?>
                <script>
                    jQuery(document).ready(function ($) {
                        $("#revolut_notice").hide();
                    });
                </script>
				<?php
			}
		}
	}

	/**
	 * Display card type
	 */
	public function get_icon() {
		$icons_str = '';

		$icons_str .= '<img src="' . WC_REVOLUT_PLUGIN_URL . '/assets/images/visa.svg" style="max-width: 40px" alt="Visa" />';
		$icons_str .= '<img src="' . WC_REVOLUT_PLUGIN_URL . '/assets/images/mastercard.svg" style="max-width: 40px" alt="MasterCard" />';

		return apply_filters( 'woocommerce_gateway_icon', $icons_str, $this->id );
	}

	/**
	 * Initialise Gateway Settings Form Fields
	 */
	public function init_form_fields() {
		$this->form_fields = array(
			'enabled'         => array(
				'title'       => __( 'Enable/Disable', 'woocommerce-gateway-revolut' ),
				'label'       => __( 'Enable Revolut', 'woocommerce-gateway-revolut' ),
				'type'        => 'checkbox',
				'description' => __( 'This controls whether or not this gateway is enabled within WooCommerce.', 'woocommerce-gateway-revolut' ),
				'default'     => 'yes',
				'desc_tip'    => true,
			),
			'title'           => array(
				'title'       => __( 'Title', 'woocommerce-gateway-revolut' ),
				'type'        => 'text',
				'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce-gateway-revolut' ),
				'default'     => __( 'Pay with card', 'woocommerce-gateway-revolut' ),
				'desc_tip'    => true,
			),
			'payment_action'  => array(
				'title'       => __( 'Payment Action', 'woocommerce-gateway-revolut' ),
				'type'        => 'select',
				'default'     => 'authorize_and_capture',
				'options'     => array(
					'authorize'             => __( 'Authorize Only', 'woocommerce-gateway-revolut' ),
					'authorize_and_capture' => __( 'Authorize and Capture', 'woocommerce-gateway-revolut' )
				),
				'description' => __( 'Select "Authorize Only" mode. This allows the payment to be captured up to 7 days after the user has placed the order (e.g. when the goods are shipped or received). 
                If not selected, Revolut will try to authorize and capture all payments.', 'woocommece-gateway-revolut' ),
				'desc_tip'    => true,
			),
			'mode'            => array(
				'title'       => __( 'Select Mode', 'woocommerce-gateway-revolut' ),
				'description' => __( 'Select mode between live mode and sandbox.', 'woocommerce-gateway-revolut' ),
				'desc_tip'    => true,
				'type'        => 'select',
				'default'     => 'live',
				'options'     => array(
					'sandbox' => __( 'Sandbox', 'woocommerce-gateway-revolut' ),
					'live'    => __( 'Live', 'woocommerce-gateway-revolut' )
				)
			),
			'api_key_sandbox' => array(
				'title'       => __( 'API Key Sandbox' ),
				'description' => __( 'API Key from your Merchant settings on Revolut.', 'woocommerce-gateway-revolut' ),
				'desc_tip'    => true,
				'type'        => 'password',
				'class'       => 'enabled-sandbox'
			),
			'api_key'         => array(
				'title'       => __( 'API Key Live', 'woocommerce-gateway-revolut' ),
				'type'        => 'password',
				'description' => __( 'API Key from your Merchant settings on Revolut.', 'woocommerce-gateway-revolut' ),
				'desc_tip'    => true,
				'default'     => '',
				'class'       => 'enabled-live'
			)
		);
	}

	/**
	 * Add script to load card form
	 */
	public function revolut_enqueue_scripts() {
		wp_enqueue_script( 'revolut-core', $this->base_url . '/embed.js', false, WC_GATEWAY_REVOLUT_VERSION, true );

		wp_enqueue_script( "jquery" );
		wp_enqueue_script( 'revolut-woocommerce', plugins_url( 'assets/js/revolut.js', WC_REVOLUT_MAIN_FILE ), array(
			'revolut-core',
			'jquery'
		), WC_GATEWAY_REVOLUT_VERSION, true );

	}

	/**
	 * Build payment fields area - including fields for logged
	 * in users, and the payment fields.
	 */
	public function payment_fields() {
		if ( ! isset( $_POST['post_data'] ) ) {
			$this->unset_revolut_public_id();
		}

		$public_id  = $this->get_revolut_public_id();
		$descriptor = new RevolutOrderDescriptor( WC()->cart->get_total( '' ), get_woocommerce_currency(), null, null );
		try {
			if ( $public_id === null ) {
				$public_id = $this->create_revolut_order( $descriptor );
			} else {
				$this->update_revolut_order( $descriptor, $public_id );
			}

			$this->set_revolut_public_id( $public_id );
			echo $this->generate_inline_revolut_form( $public_id );
		} catch ( Exception $e ) {
			$this->logError( $e );
			echo 'Something went wrong';
		}
	}

	/**
	 * Return error message
	 *
	 * @param $message
	 */
	public function logError( $message ) {
		error_log( $message );
		if ( 'yes' === $this->get_option( 'sandbox' ) || $this->enable_logging ) {
			if ( empty( $this->logger ) ) {
				$this->logger = new WC_Logger();
			}
			$this->logger->add( 'revolut', $message );
		}
	}

	/**
	 * Update Revolut order metadata
	 *
	 * @param $revolut_order_id
	 * @param $wc_order_id
	 *
	 * @throws Exception
	 */
	public function update_revolut_order_metadata( $revolut_order_id, $wc_order_id ) {
		$body = array(
			'merchant_order_ext_ref' => $wc_order_id
		);

		$json = $this->revolut_api_patch( "/orders/$revolut_order_id", $body );

		if ( ! isset( $json['public_id'] ) || ! isset( $json['id'] ) ) {
			throw new Exception( 'Something went wrong: ' . json_encode( $json, JSON_PRETTY_PRINT ) );
		}
	}

	/**
	 * Create Revolut Order
	 *
	 * @param RevolutOrderDescriptor $order_descriptor
	 *
	 * @return mixed
	 * @throws Exception
	 */
	public function create_revolut_order( RevolutOrderDescriptor $order_descriptor ) {
		$capture = $this->get_option( 'payment_action' ) == 'authorize' ? 'MANUAL' : 'AUTOMATIC';
		$body    = array(
			'amount'                 => round( $order_descriptor->amount * 100 ),
			'currency'               => $order_descriptor->currency,
			'merchant_order_ext_ref' => $order_descriptor->merchant_order_ext_ref,
			'customer_email'         => $order_descriptor->customer_email,
			'capture_mode'           => $capture
		);
		$json    = $this->revolut_api_post( '/orders', $body );

		if ( ! isset( $json['public_id'] ) || ! isset( $json['id'] ) ) {
			throw new Exception( 'Something went wrong: ' . json_encode( $json, JSON_PRETTY_PRINT ) );
		}

		global $wpdb;
		$insert_query = $wpdb->prepare( 'INSERT INTO ' . $wpdb->prefix . "wc_revolut_orders (order_id, public_id)
            VALUES (UNHEX(REPLACE(%s, '-', '')), UNHEX(REPLACE(%s, '-', '')))", array(
			$json['id'],
			$json['public_id']
		) );
		$this->query( $insert_query );

		return $json['public_id'];
	}

	/**
	 * Send post to API
	 *
	 * @param $path
	 * @param null $body
	 *
	 * @return mixed
	 * @throws Exception
	 */
	private function revolut_api_post( $path, $body = null ) {
		return $this->revolut_api_request( $path, 'POST', $body );
	}

	/**
	 * Send request to API
	 *
	 * @param $path
	 * @param $method
	 * @param null $body
	 *
	 * @return mixed
	 * @throws Exception
	 */
	private function revolut_api_request( $path, $method, $body = null ) {
		global $wp_version;
		global $woocommerce;
		$request = array(
			'headers' => array(
				'Authorization' => 'Bearer ' . $this->api_key,
				'User-Agent'    => 'Revolut Payment Gateway/' . WC_GATEWAY_REVOLUT_VERSION
				                   . ' WooCommerce/' . $woocommerce->version
				                   . ' Wordpress/' . $wp_version
				                   . ' PHP/' . PHP_VERSION,
				"Content-Type"  => "application/json"
			),
			'method'  => $method
		);

		if ( $body != null ) {
			$request['body'] = json_encode( $body );
		}

		$url           = $this->api_url . $path;
		$response      = wp_remote_request( $url, $request );
		$response_body = wp_remote_retrieve_body( $response );

		if ( is_wp_error( $response ) ) {
			throw new Exception( "Something went wrong: $method $url\n" . $response->get_error_message() );
		}

		if ( wp_remote_retrieve_response_code( $response ) >= 400 ) {
			if ( $method == 'GET' && wp_remote_retrieve_response_code( $response ) >= 500 ) {
				return json_decode( $response_body, true );
			} else {
				$this->logError( "Failed request to URL $method $url" );
				$this->logError( $response_body );
				throw new Exception( "Something went wrong: $method $url\n" . $response_body );
			}
		}

		return json_decode( $response_body, true );
	}

	/**
	 * Request query to database
	 *
	 * @param $query
	 *
	 * @return bool|int
	 * @throws Exception
	 */
	private function query( $query ) {
		global $wpdb;
		$result = $wpdb->query( $query );
		if ( $result === false ) {
			throw new Exception( $wpdb->last_error );
		}

		return $result;
	}

	/**
	 * Update Revolut Order
	 *
	 * @param RevolutOrderDescriptor $order_descriptor
	 * @param $public_id
	 *
	 * @return mixed
	 * @throws Exception
	 */
	public function update_revolut_order( RevolutOrderDescriptor $order_descriptor, $public_id ) {
		global $wpdb;
		$order_query = $wpdb->prepare( 'SELECT HEX(order_id) FROM ' . $wpdb->prefix . "wc_revolut_orders
                WHERE public_id=UNHEX(REPLACE(%s, '-', ''))", array( $public_id ) );
		$order_id    = $this->uuid_dashes( $wpdb->get_col( $order_query ) );

		$body = array(
			'amount'                 => round( $order_descriptor->amount * 100 ),
			'currency'               => $order_descriptor->currency,
			'merchant_order_ext_ref' => $order_descriptor->merchant_order_ext_ref,
			'customer_email'         => $order_descriptor->customer_email
		);

		$json = $this->revolut_api_patch( "/orders/$order_id", $body );

		if ( ! isset( $json['public_id'] ) || ! isset( $json['id'] ) ) {
			throw new Exception( 'Something went wrong: ' . json_encode( $json, JSON_PRETTY_PRINT ) );
		}

		return $json['public_id'];
	}

	/**
	 * Replace dashes
	 *
	 * @param $uuid
	 *
	 * @return string|string[]|null
	 */
	private function uuid_dashes( $uuid ) {
		if ( is_array( $uuid ) ) {
			$uuid = $uuid[0];
		}

		$result = preg_replace( '/(\w{8})(\w{4})(\w{4})(\w{4})(\w{12})/i', '$1-$2-$3-$4-$5', $uuid );

		return $result;
	}

	/**
	 * Revolut API patch
	 *
	 * @param $path
	 * @param $body
	 *
	 * @return mixed
	 * @throws Exception
	 */
	private function revolut_api_patch( $path, $body ) {
		return $this->revolut_api_request( $path, 'PATCH', $body );
	}

	/**
	 * Unset Revolut public_id
	 */
	private function unset_revolut_public_id() {
		WC()->session->__unset( 'revolut_public_id' );
	}

	/**
	 * Set Revolut public_id
	 *
	 * @param $value
	 */
	private function set_revolut_public_id( $value ) {
		WC()->session->set( 'revolut_public_id', $value );
	}

	/**
	 * Get Revolut public_id
	 * @return array|string|null
	 */
	private function get_revolut_public_id() {
		return WC()->session->get( 'revolut_public_id' );
	}

	/**
	 * Add public_id field and logo on card form
	 *
	 * @param $public_id
	 *
	 * @return string
	 */
	public function generate_inline_revolut_form( $public_id ) {
		return '<input type="hidden" name="revolut_public_id" value="' . $public_id . '"/>'
		       . '<div id="woocommerce-revolut-card-element" data-public-id="' . $public_id . '" style="height: 40px;"></div>'
		       . '<img src="' . WC_REVOLUT_PLUGIN_URL . '/assets/images/Powered_by_Revolut.svg" style="float: left;" alt="power_by_revolut" />';
	}

	/**
	 * Process the payment and return the result.
	 */
	public function process_payment( $wc_order_id ) {
		$wc_order = wc_get_order( $wc_order_id );

		try {
			global $wpdb;

			// find public_id
			$public_id = $this->get_revolut_public_id();
			if ( $public_id == null ) {
				throw new Exception( 'Missing revolut_public_id parameter' );
			}

			// resolve into order_id
			$order_query = $wpdb->prepare( 'SELECT HEX(order_id) FROM ' . $wpdb->prefix . "wc_revolut_orders
                WHERE public_id=UNHEX(REPLACE(%s, '-', ''))", array( $public_id ) );
			$order_id    = $this->uuid_dashes( $wpdb->get_col( $order_query ) );

			// verify that the order was paid
			$check_state = false;
			for ( $i = 0; $i <= 9; $i ++ ) {
				$order = $this->revolut_api_get( '/orders/' . $order_id );
				if ( $order['state'] == 'COMPLETED' || $order['state'] == 'AUTHORISED' || $order['state'] == "IN_SETTLEMENT" ) {
					try {
						$this->update_revolut_order_metadata( $order_id, $wc_order_id );
						$check_state = true;
						break;	
					} catch ( Exception $e) {}
					
				}
				if ( $order['state'] == 'PENDING' ) {
					throw new Exception( 'Something went wrong while taking the payment. Payment should be retried.' );
				} else if ( $i == 9 && $check_state == false ) {
					$wc_order->update_status( 'on-hold' );
					$wc_order->add_order_note( 'Something went wrong during the checkout process. Order status changed 
					to "On Hold" to allow manual review. Please check your Revolut account to verify that this payment was taken. 
					You might need to contact your customer if it wasn\'t. (Order ID: ' . $order_id . ').' );
					$this->unset_revolut_public_id();
					if ( isset( WC()->cart ) ) {
						WC()->cart->empty_cart();
					}

					return array(
						'result'   => 'success',
						'redirect' => $this->get_return_url( $wc_order ),
					);
				}
				sleep( 2 );
			}


			if ( $check_state == true ) {
				if ( $order['merchant_order_ext_ref'] != null ) {
					throw new Exception( 'Invalid revolut_public_id' );
				}

				// update internal table to avoid piggybacking on already paid order
				$update_query = $wpdb->prepare( 'UPDATE ' . $wpdb->prefix . "wc_revolut_orders
                SET wc_order_id=%d
                WHERE public_id=UNHEX(REPLACE(%s, '-', '')) AND wc_order_id IS NULL",
					array( $wc_order_id, $public_id ) );
				$updated_rows = $this->query( $update_query );

				if ( $updated_rows !== 1 ) {
					throw new Exception( "Invalid revolut_public_id, updated rows: $updated_rows" );
				}

				if ( $order['state'] == 'AUTHORISED' ) {
					$wc_order->update_status( 'on-hold' );
					$wc_order->add_order_note( 'Payment amount has been authorised. Order status changed to "On Hold" to allow manual review. (Order ID: ' . $order_id . ').' );
					$this->unset_revolut_public_id();
					if ( isset( WC()->cart ) ) {
						WC()->cart->empty_cart();
					}
				} else {
					$wc_order->payment_complete( $order_id );
					$wc_order->add_order_note( 'Payment amount has been captured successfully. (Order ID: ' . $order_id . ').' );
					$this->unset_revolut_public_id();
					if ( isset( WC()->cart ) ) {
						WC()->cart->empty_cart();
					}
				}

				return array(
					'result'   => 'success',
					'redirect' => $this->get_return_url( $wc_order ),
				);
			}
		} catch ( Exception $e ) {
			$this->logError( $e );
			$wc_order->update_status( 'failed' );

			return array(
				'result'   => 'fail',
				'redirect' => '',
			);
		}
	}

	/**
	 * Get data from API
	 *
	 * @param $path
	 *
	 * @return mixed
	 * @throws Exception
	 */
	private function revolut_api_get( $path ) {
		return $this->revolut_api_request( $path, 'GET' );
	}

	/**
	 * Add admin notice when use revolut payment without API key
	 */
	public function admin_notices() {
		if ( 'yes' !== $this->get_option( 'enabled' )
		     || ! empty( $this->api_key ) ) {
			return;
		}

		if ( empty( $this->api_key ) && empty( $this->api_key_sandbox ) ) {
			echo '<div class="error revolut-passphrase-message"><p>'
			     . __( 'Revolut requires an API Key to work.', 'woocommerce-gateway-revolut' )
			     . '</p></div>';
		}
	}
}

class RevolutOrderDescriptor {
	public $amount;
	public $currency;
	public $merchant_order_ext_ref;
	public $customer_email;

	/**
	 * OrderDescriptor constructor.
	 *
	 * @param float $amount
	 * @param string $currency
	 * @param string $merchant_order_ext_ref
	 * @param string $customer_email
	 */
	public function __construct( $amount, $currency, $merchant_order_ext_ref, $customer_email ) {
		$this->amount                 = $amount;
		$this->currency               = $currency;
		$this->merchant_order_ext_ref = $merchant_order_ext_ref;
		$this->customer_email         = $customer_email;
	}
}

/**
 * Revolut Order Descriptor
 */
function RevolutOrderDescriptor( WC_Order $order ) {
	return new RevolutOrderDescriptor( $order->get_total(), $order->get_currency(), $order->get_id(), $order->get_billing_email() );
}
